]> git.sesse.net Git - nageru/blobdiff - mixer.h
Add a HTTP server for stream output.
[nageru] / mixer.h
diff --git a/mixer.h b/mixer.h
index c4697310ec95757fecbfacea767ed887ba821e03..a177854b46cde0229ebfcb669998afea709813b7 100644 (file)
--- a/mixer.h
+++ b/mixer.h
@@ -6,13 +6,18 @@
 #include <epoxy/gl.h>
 #undef Success
 #include <movit/effect_chain.h>
+#include <movit/flat_input.h>
 #include <functional>
 
-#include "bmusb.h"
+#include "bmusb/bmusb.h"
 #include "h264encode.h"
 #include "pbo_frame_allocator.h"
 #include "ref_counted_frame.h"
 #include "ref_counted_gl_sync.h"
+#include "theme.h"
+#include "resampler.h"
+#include "timebase.h"
+#include "httpd.h"
 
 #define NUM_CARDS 2
 
@@ -30,22 +35,39 @@ public:
        void start();
        void quit();
 
-       enum Source {
-               SOURCE_INPUT1,
-               SOURCE_INPUT2,
-               SOURCE_SBS,
-       };
-       void cut(Source source);
+       void transition_clicked(int transition_num);
+       void channel_clicked(int preview_num);
 
        enum Output {
                OUTPUT_LIVE = 0,
                OUTPUT_PREVIEW,
+               OUTPUT_INPUT0,
+               OUTPUT_INPUT1,
+               OUTPUT_INPUT2,
+               OUTPUT_INPUT3,
                NUM_OUTPUTS
        };
 
        struct DisplayFrame {
-               GLuint texnum;
-               RefCountedGLsync ready_fence;  // Asserted when the texture is done rendering.
+               // The chain for rendering this frame. To render a display frame,
+               // first wait for <ready_fence>, then call <setup_chain>
+               // to wire up all the inputs, and then finally call
+               // chain->render_to_screen() or similar.
+               movit::EffectChain *chain;
+               std::function<void()> setup_chain;
+
+               // Asserted when all the inputs are ready; you cannot render the chain
+               // before this.
+               RefCountedGLsync ready_fence;
+
+               // Holds on to all the input frames needed for this display frame,
+               // so they are not released while still rendering.
+               std::vector<RefCountedFrame> input_frames;
+
+               // Textures that should be released back to the resource pool
+               // when this frame disappears, if any.
+               // TODO: Refcount these as well?
+               std::vector<GLuint> temp_textures;
        };
        // Implicitly frees the previous one if there's a new frame available.
        bool get_display_frame(Output output, DisplayFrame *frame) {
@@ -58,10 +80,9 @@ public:
                output_channel[output].set_frame_ready_callback(callback);
        }
 
-       // Ignored for OUTPUT_LIVE.
-       void set_preview_size(Output output, int width, int height)
+       std::vector<std::string> get_transition_names()
        {
-               output_channel[output].set_size(width, height);
+               return theme->get_transition_names(pts());
        }
 
 private:
@@ -72,39 +93,42 @@ private:
        void thread_func();
        void subsample_chroma(GLuint src_tex, GLuint dst_dst);
        void release_display_frame(DisplayFrame *frame);
+       double pts() { return double(pts_int) / TIMEBASE; }
+
+       HTTPD httpd;
 
        QSurface *mixer_surface, *h264_encoder_surface;
        std::unique_ptr<movit::ResourcePool> resource_pool;
-       std::unique_ptr<movit::EffectChain> chain;
-       std::unique_ptr<movit::EffectChain> preview_chain;
+       std::unique_ptr<Theme> theme;
+       std::unique_ptr<movit::EffectChain> display_chain;
        GLuint cbcr_program_num;  // Owned by <resource_pool>.
        std::unique_ptr<H264Encoder> h264_encoder;
 
-       // Effects part of <chain>. Owned by <chain>.
-       movit::YCbCrInput *input[NUM_CARDS];
-       movit::Effect *resample_effect, *resample2_effect;
-       movit::Effect *padding_effect, *padding2_effect;
-
-       // Effects part of <preview_chain>. Owned by <preview_chain>.
-       movit::YCbCrInput *preview_input;
+       // Effects part of <display_chain>. Owned by <display_chain>.
+       movit::FlatInput *display_input;
 
-       Source current_source = SOURCE_INPUT1;
-       int frame = 0;
+       int64_t pts_int = 0;  // In TIMEBASE units.
 
        std::mutex bmusb_mutex;
        struct CaptureCard {
                BMUSBCapture *usb;
                std::unique_ptr<PBOFrameAllocator> frame_allocator;
 
-               // Threading stuff
-               bool thread_initialized = false;
+               // Stuff for the OpenGL context (for texture uploading).
                QSurface *surface;
                QOpenGLContext *context;
 
-               bool new_data_ready = false;  // Whether new_frame contains anything.
+               bool new_data_ready = false;  // Whether new_frame and new_frame_audio contains anything.
+               bool should_quit = false;
                RefCountedFrame new_frame;
                GLsync new_data_ready_fence;  // Whether new_frame is ready for rendering.
+               std::vector<float> new_frame_audio;
                std::condition_variable new_data_ready_changed;  // Set whenever new_data_ready is changed.
+               unsigned dropped_frames = 0;  // Before new_frame.
+
+               std::mutex audio_mutex;
+               std::unique_ptr<Resampler> resampler;  // Under audio_mutex.
+               int last_timecode = -1;  // Unwrapped.
        };
        CaptureCard cards[NUM_CARDS];  // protected by <bmusb_mutex>
 
@@ -112,10 +136,10 @@ private:
 
        class OutputChannel {
        public:
-               void output_frame(GLuint tex, RefCountedGLsync fence);
+               ~OutputChannel();
+               void output_frame(DisplayFrame frame);
                bool get_display_frame(DisplayFrame *frame);
                void set_frame_ready_callback(new_frame_ready_callback_t callback);
-               void set_size(int width, int height);  // Ignored for OUTPUT_LIVE.
 
        private:
                friend class Mixer;
@@ -126,8 +150,6 @@ private:
                bool has_current_frame = false, has_ready_frame = false;  // protected by <frame_mutex>
                new_frame_ready_callback_t new_frame_ready_callback;
                bool has_new_frame_ready_callback = false;
-
-               int width = 1280, height = 720;
        };
        OutputChannel output_channel[NUM_OUTPUTS];